/*********************************************************************
 * Desktop Station AUTO Sketch for DS shield series
 *
 * Copyright (C) 2017 Yaasan
 *
 */

// include the library code:
#include <EEPROM.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "DSCoreM_Type.h"
#include "DSCoreM_Common.h"
#include "DSCoreM_List.h"
#include "DSCoreM_DCC.h"
#include "DSCoreM_MM2.h"
#include "DSCoreM.h"
#include "Functions.h"
#include "NewSerial.h"
#include "TrackReporterS88_DS.h"
#include "TimerOne_x.h"

#define MAX_S88DECODER 1

#define LEDSTATE_STOP 0
#define LEDSTATE_RUN 1
#define LEDSTATE_ERR 2

#define PIN_RUNLED	2

#define MODE_SPEEDSTEP128 2

#define THRESHOLD_EDC 276  // 1024d * (1.2 / 8) * 9V / 5V
#define THRESHOLD_OV  614  // 1024d * (1.2 / 8) * 20V / 5V

NewSerial SerialDS;

/* Decralation classes */
DSCoreLib DSCore;
TrackReporterS88_DS reporter(MAX_S88DECODER);

/* LED functions */
byte gLED_State = LEDSTATE_STOP;
byte gLED_Toggle = 0;

//Task Schedule
unsigned long gPreviousL8 = 0; // 1000ms interval


//コマンド関数定義
void CMD_Wait(word inWaitMs);
void CMD_Power(byte inState);
void CMD_LocSpeed(word inAddress, word inSpeed);
void CMD_LocDirection(word inAddress, byte inDirection);
void CMD_LocFunction(word inAddress, byte inFunctionNo, byte inFunctionState);
void CMD_LocFunctionMomentary(word inAddress, byte inFunctionNo);
void CMD_Turnout(word inAddress, byte inDirection);


/***********************************************************************
* 
* Boot setup
* 
************************************************************************/

void setup()
{
	DSCore.Init();

	//Pin mode
	pinMode(A1, INPUT);//Edc
	
	/* LED turn on */
	changeLED(0);
	
	/* Clear data of the S88 decoder */
	reporter.refresh();
	
	/* Debug message for PC */
	SerialDS.Init();
	SerialDS.println("--------------------------------------");
	SerialDS.println("Desktop Station shield R1 2017/11/12  ");
	SerialDS.println("--------------------------------------");
	SerialDS.println("100 Ready");
	
	//Reset task
	gPreviousL8 = millis();
	
	Timer1.initialize(20);// 5ms interval
	Timer1.attachInterrupt( timerIsr ); // attach the service routine here
	
	sei();    //ENABLE INTERRUPTION
	
}

/***********************************************************************
* 
* Command for locomotives and turnouts
* 
************************************************************************/

void CMD_Wait(word inWaitMs)
{
	unsigned long aTemp = millis() + inWaitMs;
	
	while ( aTemp > millis() )
	{
		//NOP
		__asm__("nop\n\t");
	}
	
}

void CMD_Power(byte inState)
{
	/* LED状態の変更 */
	changeLED(inState);
	
	DSCore.SetPower(inState);
}

void CMD_LocSpeed(word inAddress, word inSpeed)
{
	DSCore.SetLocoSpeedEx(inAddress, inSpeed, MODE_SPEEDSTEP128);
}

void CMD_LocDirection(word inAddress, byte inDirection)
{
	DSCore.SetLocoDirection(inAddress, inDirection);
}

void CMD_LocFunction(word inAddress, byte inFunctionNo, byte inFunctionState)
{
	DSCore.SetLocoFunction(inAddress, inFunctionNo + 1, inFunctionState);
}

void CMD_LocFunctionMomentary(word inAddress, byte inFunctionNo)
{
	DSCore.SetLocoFunction(inAddress, inFunctionNo + 1, 1);
	CMD_Wait(10);
	DSCore.SetLocoFunction(inAddress, inFunctionNo + 1, 0);
}


void CMD_Turnout(word inAddress, byte inDirection)
{
	DSCore.SetTurnout(inAddress, inDirection);
}

/***********************************************************************
* 
* Timer Interval
* 
************************************************************************/

void timerIsr()
{
	uint8_t aErrNo = 0;
	word aEdc;
	aErrNo = 0;
	
	//パルスジェネレータのスキャン
	DSCore.Scan();
	
	if( (millis() - gPreviousL8) >= 200)
	{
		//Reset task
		gPreviousL8 = millis();
		
		//LED Control
		taskLED();
		
		/* Sensor check */
		aEdc = analogRead(A1);
		
		/* OV check */
		if( aEdc > THRESHOLD_OV )
		{
			aErrNo = 2;
		}
		/* LV check */
		if( aEdc < THRESHOLD_EDC )
		{
			aErrNo = 1;
			
		}
		/* エラー処理 */
		if( aErrNo > 0)
		{
			if( DSCore.IsPower() == 1)
			{
				PowerOffByErr(aErrNo);
			}
			
		}
	}
	
}

/***********************************************************************
* 
* Main Loop
* 
************************************************************************/

void loop()
{
	
	user_program();
	
}

/***********************************************************************
* 
* RUN LED Control functions
* 
************************************************************************/

void PowerOffByErr(uint8_t inErrNo)
{
	DSCore.SetPower(0);
	
	/* 状態変更 */
	gLED_State = LEDSTATE_ERR;
}


void changeLED(byte inState)
{
	if( inState == 1)
	{
		/* 状態変更 */
		gLED_State = LEDSTATE_RUN;
	}
	else
	{
		/* 状態変更 */
		gLED_State = LEDSTATE_STOP;
	}
	
	switch( gLED_State)
	{
	case LEDSTATE_STOP:
		digitalWrite(PIN_RUNLED, 0);
		
		break;
		
	case LEDSTATE_RUN:
		digitalWrite(PIN_RUNLED, 1);
		break;
		
	case LEDSTATE_ERR:
		break;
	}
}

void taskLED(void)
{
	
	
	switch( gLED_State)
	{
	case LEDSTATE_STOP:
		break;
		
	case LEDSTATE_RUN:
		break;
		
	case LEDSTATE_ERR:
		digitalWrite(PIN_RUNLED, gLED_Toggle);
		gLED_Toggle = (~gLED_Toggle) & 1;
		
		break;
	}
}

/*****************************************************************/
/*  User Program Area                                            */
/*****************************************************************/

void user_program(void)
{
	
	SerialDS.println("Power On");
	CMD_Power(1);
	
	CMD_LocFunction(ADDR_DCC + 3, 0, 1);
	CMD_LocFunction(ADDR_DCC + 3, 1, 1);
	CMD_Wait(500);
	
	SerialDS.println("Loc 3, Func 10, On");
	CMD_LocFunction(ADDR_DCC + 3, 10, 1);
	
	CMD_Wait(500);
	
	SerialDS.println("Loc 3, Func 11, On");
	CMD_LocFunction(ADDR_DCC + 3, 11, 1);
	
	CMD_Wait(500);
	
	SerialDS.println("Loc 3, Func 9, On");
	CMD_LocFunction(ADDR_DCC + 3, 9, 1);
	
	CMD_Wait(500);
	
	SerialDS.println("End...");
	/* 永久ループ（終了） */
	while(1){}
	
}

